SystemOrganization addCategory: #'Zippers-Support'!
SystemOrganization addCategory: #Zippers!
SystemOrganization addCategory: #'Zippers-BST'!
SystemOrganization addCategory: #'Zippers-Tests'!

Error subclass: #ZipperNavigation
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers'!

!ZipperNavigation commentStamp: 'fbs 12/28/2010 17:36' prior: 0!
Raise me whenever a Zipper tries to navigate to somewhere it cannot - past the end of a list, to a missing subtree, etc.!

TestCase subclass: #BinarySearchTreeTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers-Tests'!

!BinarySearchTreeTest methodsFor: 'testing - adding' stamp: 'fbs 2/25/2011 13:33'!
testAddToEmptyTreeAdds
	| items tree |
	tree := (BinarySearchTree tree: ZTree empty) add: 1.
	items := OrderedCollection new.
	PreOrderTraversal new traverse: tree doing: [:each | items add: each].
	
	self assert: #(1) asOrderedCollection = items.! !

!BinarySearchTreeTest methodsFor: 'testing - adding' stamp: 'fbs 2/25/2011 16:32'!
testAddingLargerPreservesOrder
	"The in-order traversal of a binary tree always yields a sorted collection."
	| items tree |
	tree := (BinarySearchTree with: 2) add: 3.
	items := OrderedCollection new.
	InOrderTraversal new traverse: tree doing: [:each | items add: each].
	
	self assert: #(2 3) asOrderedCollection = items.! !

!BinarySearchTreeTest methodsFor: 'testing - adding' stamp: 'fbs 2/25/2011 16:32'!
testAddingPreservesOrder
	"The in-order traversal of a binary tree always yields a sorted collection."
	| items tree |
	tree := ((BinarySearchTree with: 1) add: 3) add: 2.
	items := OrderedCollection new.
	InOrderTraversal new traverse: tree doing: [:each | items add: each].
	
	self assert: #(1 2 3) asOrderedCollection = items.! !

!BinarySearchTreeTest methodsFor: 'testing - adding' stamp: 'fbs 2/25/2011 16:32'!
testAddingSmallerPreservesOrder
	"The in-order traversal of a binary tree always yields a sorted collection."
	| items tree |
	tree := (BinarySearchTree with: 2) add: 1.
	items := OrderedCollection new.
	InOrderTraversal new traverse: tree doing: [:each | items add: each].
	
	self assert: #(1 2) asOrderedCollection = items.! !

!BinarySearchTreeTest methodsFor: 'testing - enumerating' stamp: 'fbs 2/26/2011 22:50'!
testCollectExecutesBlockOncePerNode
	| items t |
	t := ((BinarySearchTree with: 2) add: 3) add: 1.
	items := t collect: [:each | each * 2].
	self assert: items = (#(1 2 3) * 2) asOrderedCollection.! !

!BinarySearchTreeTest methodsFor: 'testing - adding' stamp: 'fbs 2/24/2011 17:32'!
testNewTreeIsBlank
	self assert: BinarySearchTree new asOrderedCollection = #() asOrderedCollection.! !

!BinarySearchTreeTest methodsFor: 'testing - converting' stamp: 'fbs 4/27/2011 21:29'!
testRebalance
	| bst zipper |
	bst := BinarySearchTree tree: ZTree empty.
	"Construct a degenerate BST."
	1 to: 10 do: [:i | bst := bst add: i].
	
	"We don't check the entire structure, but we know it should look like this:"
	"(7 (4 (2 (1), (3)), (6 (5))), (9 (8), (10)))"
	zipper := bst rebalanced zipper.
	self assert: zipper value = 7.
	self assert: zipper down value = 4.
	self assert: zipper down right value = 9.! !

!BinarySearchTreeTest methodsFor: 'testing - instance creation' stamp: 'fbs 2/24/2011 22:52'!
testWithReturnsSingleNodeTree
	| bst value |
	value := 1.
	bst := BinarySearchTree with: value.
	self assert: (bst collect: [:each | each]) = #(1) asOrderedCollection.
	! !

TestCase subclass: #ListZipperTest
	instanceVariableNames: 'prefix suffix'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers-Tests'!

!ListZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 12/30/2010 21:13'!
testChangeChangesValue
	| list zipper |
	list := #(1 2 3) asOrderedCollection.
	zipper := ListZipper on: list.
	zipper := zipper right right.
	zipper := zipper change: [:x | x + 1].
	
	zipper := zipper left left.
	
	self assert: zipper focus = #(1 2 4) asOrderedCollection.! !

!ListZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 12/30/2010 21:13'!
testChangePreservesOriginal
	| original list zipper |
	original := list := #(1 (2 3) 4) asOrderedCollection.
	zipper := ListZipper on: list.
	zipper := zipper right.
	zipper := zipper change: [:x | x size].
	zipper := zipper left.
	
	self assert: zipper focus = #(1 2 4) asOrderedCollection.
	self assert: list = original description: 'Zipper not functional'.! !

!ListZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 12/30/2010 21:15'!
testChangeSharesStructure
	| original list zipper |
	original := list := #(1 (2 3) 4) asOrderedCollection.
	zipper := ListZipper on: list.
	zipper := zipper right right.
	zipper := zipper change: [:x | x + 1].
	zipper := zipper left left.
	
	self assert: zipper focus = #(1 (2 3) 5) asOrderedCollection.
	self assert: zipper focus second == original second description: 'Zipper not sharing original structure'.! !

!ListZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 12/28/2010 16:50'!
testLeft
	| left list zipper right |
	list := #(1 2 3) asOrderedCollection.
	zipper := ListZipper on: list.
	right := zipper right right.
	
	left := right left.
	self assert: left focus = list allButFirst.
	
	left := left left.
	self assert: left focus = list.! !

!ListZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 12/30/2010 22:17'!
testLeftPastBeginningOfCollectionSignalsException
	| zipper |
	zipper := ListZipper on: #(1 2).
	self
		should: [ zipper right left left ]
		raise: ZipperNavigation.! !

!ListZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 12/28/2010 16:49'!
testOn
	| list zipper |
	list := #(1 2 3 4 5).
	zipper := ListZipper on: list.
	self assert: zipper focus = list asOrderedCollection.! !

!ListZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 12/28/2010 16:50'!
testRight
	| list zipper right |
	list := #(1 2 3) asOrderedCollection.
	zipper := ListZipper on: list.
	right := zipper right.
	self assert: right focus = list allButFirst.
	
	right := right right.
	self assert: right focus = list allButFirst allButFirst.! !

!ListZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 12/30/2010 22:17'!
testRightPastEndOfCollectionSignalsException
	| zipper |
	zipper := ListZipper on: #(1 2).
	self
		should: [ zipper right right right ]
		raise: ZipperNavigation.! !

TestCase subclass: #TraversalTestCase
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers-Tests'!

TraversalTestCase subclass: #InOrderTraversalTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers-Tests'!

!InOrderTraversalTest methodsFor: 'running' stamp: 'fbs 2/24/2011 17:37'!
makeTraversal
	^ InOrderTraversal new.! !

!InOrderTraversalTest methodsFor: 'testing - enumerating' stamp: 'fbs 2/28/2011 17:46'!
testCollectEnumeratesCorrectly
	| i tree |
	tree := ZTree value: 1 children: {ZTree value: 2 children: {ZTree value: 3. ZTree value: 4}. ZTree value: 5}.
	i := self makeTraversal traverse: tree collecting: [:each | each].
	
	self assert: i asOrderedCollection = #(3 2 4 1 5) asOrderedCollection.! !

!InOrderTraversalTest methodsFor: 'testing - enumerating' stamp: 'fbs 4/24/2011 22:04'!
testTraversalHandlesEmptySubtrees
	| i tree |
	tree := ZTree value: 1 children: {ZTree empty. ZTree value: 2}.
	i := self makeTraversal traverseNodes: tree collecting: #value.
	
	self assert: i asOrderedCollection = #(1 2) asOrderedCollection.! !

TraversalTestCase subclass: #PostOrderTraversalTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers-Tests'!

!PostOrderTraversalTest methodsFor: 'running' stamp: 'fbs 2/18/2011 07:24'!
makeTraversal
	^ PostOrderTraversal new.! !

!PostOrderTraversalTest methodsFor: 'testing - enumerating' stamp: 'fbs 2/28/2011 17:47'!
testCollectCollectsCorrectly
	| i tree |
	tree := ZTree value: 1 children: {ZTree value: 2 children: {ZTree value: 3. ZTree value: 4}. ZTree value: 5}.
	i := self makeTraversal traverse: tree collecting: [:each | each * 2].
	
	self assert: i asOrderedCollection = (#(3 4 2 5 1) * 2) asOrderedCollection.! !

!PostOrderTraversalTest methodsFor: 'testing - enumerating' stamp: 'fbs 2/28/2011 17:47'!
testCollectEnumeratesCorrectly
	| i tree |
	tree := ZTree value: 1 children: {ZTree value: 2 children: {ZTree value: 3. ZTree value: 4}. ZTree value: 5}.
	i := self makeTraversal traverse: tree collecting: [:each | each].
	
	self assert: i asOrderedCollection = #(3 4 2 5 1) asOrderedCollection.! !

!PostOrderTraversalTest methodsFor: 'testing - enumerating' stamp: 'fbs 4/24/2011 21:30'!
testCollectNodesEnumeratesCorrectly
	| i tree |
	tree := ZTree value: 1 children: {ZTree value: 2 children: {ZTree value: 3. ZTree value: 4}. ZTree value: 5}.
	i := self makeTraversal traverseNodes: tree collecting: #value.
	
	self assert: i asOrderedCollection = #(3 4 2 5 1) asOrderedCollection.! !

!PostOrderTraversalTest methodsFor: 'testing - enumerating' stamp: 'fbs 4/24/2011 21:35'!
testTraversalHandlesEmptySubtrees
	| i tree |
	tree := ZTree value: 1 children: {ZTree empty. ZTree value: 2}.
	i := self makeTraversal traverseNodes: tree collecting: #value.
	
	self assert: i asOrderedCollection = #(2 1) asOrderedCollection.! !

!PostOrderTraversalTest methodsFor: 'testing - enumerating' stamp: 'fbs 2/18/2011 07:26'!
testTreeTraversalMovesDepthFirst
	self
		assertTraversalOf: (ZTree value: 1 children: {ZTree value: 2 children: {ZTree value: 3. ZTree value: 4}. ZTree value: 5})
		 yieldsOrder: #(3 4 2 5 1).! !

!PostOrderTraversalTest methodsFor: 'testing - enumerating' stamp: 'fbs 2/18/2011 07:51'!
testTreeTraversalMovesDepthFirstOnTriangleTree
	self
		assertTraversalOf: (ZTree value: 1 children: {ZTree value: 2. ZTree value: 3})
		 yieldsOrder: #(2 3 1).! !

TraversalTestCase subclass: #PreOrderTraversalTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers-Tests'!

!PreOrderTraversalTest methodsFor: 'running' stamp: 'fbs 2/17/2011 22:04'!
makeTraversal
	^ PreOrderTraversal new.! !

!PreOrderTraversalTest methodsFor: 'testing - enumerating' stamp: 'fbs 2/28/2011 17:47'!
testCollectCollectsCorrectly
	| i tree |
	tree := ZTree value: 1 children: {ZTree value: 2 children: {ZTree value: 3. ZTree value: 4}. ZTree value: 5}.
	i := self makeTraversal traverse: tree collecting: [:each | each * 2].
	
	self assert: i asOrderedCollection = (#(1 2 3 4 5) * 2) asOrderedCollection.! !

!PreOrderTraversalTest methodsFor: 'testing - enumerating' stamp: 'fbs 4/24/2011 21:56'!
testTraversalHandlesEmptySubtrees
	| i tree |
	tree := ZTree value: 1 children: {ZTree empty. ZTree value: 2}.
	i := self makeTraversal traverseNodes: tree collecting: #value.
	
	self assert: i asOrderedCollection = #(1 2) asOrderedCollection.! !

!PreOrderTraversalTest methodsFor: 'testing - enumerating' stamp: 'fbs 2/17/2011 22:11'!
testTreeTraversalMovesDepthFirst
	self
		assertTraversalOf: (ZTree value: 1 children: {ZTree value: 2 children: {ZTree value: 3. ZTree value: 4}. ZTree value: 5})
		 yieldsOrder: #(1 2 3 4 5).! !

!TraversalTestCase class methodsFor: 'testing' stamp: 'fbs 2/26/2011 22:55'!
isAbstract
	^ self name = #TraversalTestCase.! !

!TraversalTestCase methodsFor: 'testing' stamp: 'fbs 2/17/2011 22:08'!
assertTraversalOf: aTree yieldsOrder: aCollection
	| i |
	i := #() asOrderedCollection.
	self makeTraversal traverse: aTree doing: [:each | i add: each ].
	
	self assert: i = aCollection asOrderedCollection.! !

!TraversalTestCase methodsFor: 'running' stamp: 'fbs 2/17/2011 22:04'!
makeTraversal
	^ self subclassResponsibility.! !

!TraversalTestCase methodsFor: 'running' stamp: 'fbs 2/17/2011 22:04'!
setUp
	traversal := self makeTraversal.! !

!TraversalTestCase methodsFor: 'testing - enumerating' stamp: 'fbs 2/17/2011 22:05'!
testEmptyTraversalDoesNothing
	| i |
	i := 0.
	self makeTraversal traverse: (ZTree value: 1) doing: [:each | i := i + 1].
	
	self assert: i = 1.! !

!TraversalTestCase methodsFor: 'testing - enumerating' stamp: 'fbs 2/28/2011 17:45'!
testEnumeratingEmptyTreeReturnsEmptyCollection
	| items |
	items := self makeTraversal traverse: ZTree empty collecting: [:each | each ].
	self assert: items isEmpty.! !

!TraversalTestCase methodsFor: 'testing - enumerating' stamp: 'fbs 2/17/2011 22:05'!
testTraverseBranchingTreeCallsBlock
	| i |
	i := 0.
	self makeTraversal traverse: (ZTree value: 0 children: {ZTree value: 1. ZTree value: 2}) doing: [:each | i := i + 1].
	
	self assert: i = 3.! !

!TraversalTestCase methodsFor: 'testing - enumerating' stamp: 'fbs 2/17/2011 22:06'!
testTreeTraversalCallsBlock
	| i |
	i := 0.
	self makeTraversal traverse: (ZTree value: 1) doing: [:each | i := i + 1].
	
	self assert: i = 1.! !

TestCase subclass: #TreeTestCases
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers-Tests'!

!TreeTestCases class methodsFor: 'testing' stamp: 'fbs 2/26/2011 22:55'!
isAbstract
	^ self name = #TreeTestCases.! !

!TreeTestCases methodsFor: 'fixtures' stamp: 'fbs 2/25/2011 17:35'!
emptyTree
	^ ZTree empty.! !

!TreeTestCases methodsFor: 'fixtures' stamp: 'fbs 2/25/2011 08:00'!
flatTree
	" Best viewed in a monospaced font:
	       1
	   / / | \ \
	  2 3  4 5 6"
	^ ZTree value: 1 children: ((2 to: 6) collect: [ :i | ZTree value: i])! !

!TreeTestCases methodsFor: 'fixtures' stamp: 'fbs 2/25/2011 08:00'!
nestedTriangleTree
	" Best viewed in a monospaced font:
	    1
	   / \
	  2   3 
	     / \
	    4   5 "
	| b c |
	b := ZTree value: 2.
	c := ZTree value: 3 children: {ZTree value: 4. ZTree value: 5}.
	^ ZTree value: 1 children: {b. c.}! !

!TreeTestCases methodsFor: 'fixtures' stamp: 'fbs 2/25/2011 07:59'!
triangleTree
	" Best viewed in a monospaced font:
	   1
	  / \
	 2   3 "
	| b c |
	b := ZTree value: 2.
	c := ZTree value: 3.
	^ ZTree value: 1 children: {b. c.}! !

TreeTestCases subclass: #TreeZipperTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers-Tests'!

!TreeZipperTest methodsFor: 'fixtures' stamp: 'fbs 2/25/2011 17:36'!
emptyTree
	^ ZTree empty.! !

!TreeZipperTest methodsFor: 'fixtures' stamp: 'fbs 12/31/2010 18:21'!
flatTree
	" Best viewed in a monospaced font:
	       1
	   / / | \ \
	  2 3  4 5 6"
	^ ZTree value: 1 children: ((2 to: 6) collect: [ :i | ZTree value: i])! !

!TreeZipperTest methodsFor: 'fixtures' stamp: 'fbs 2/25/2011 08:05'!
makeZipperOn: aTree
	^ TreeZipper on: aTree.! !

!TreeZipperTest methodsFor: 'fixtures' stamp: 'fbs 12/31/2010 17:58'!
nestedTriangleTree
	" Best viewed in a monospaced font:
	    1
	   / \
	  2   3 
	     / \
	    4   5 "
	| b c |
	b := ZTree value: 2.
	c := ZTree value: 3 children: {ZTree value: 4. ZTree value: 5}.
	^ ZTree value: 1 children: {b. c.}! !

!TreeZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 12/31/2010 18:47'!
testChangeReplacesValue
	| tree zipper |
	tree := self triangleTree.
	zipper := (TreeZipper on: tree) down.
	
	zipper := zipper change: [ :i | i + 1].
	self assert: zipper focus value = 3 description: 'No change made'.
	self assert: zipper right focus value = 3 description: 'Position changed'.
	self assert: zipper up focus value = 1 description: 'Position changed'.
	! !

!TreeZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 12/31/2010 19:31'!
testChangeToReplacesValue
	| tree zipper |
	tree := self triangleTree.
	zipper := (TreeZipper on: tree) down.
	
	zipper := zipper changeTo: 5.
	self assert: zipper focus value = 5 description: 'No change made'.
	self assert: zipper right focus value = 3 description: 'Position changed'.
	self assert: zipper up focus value = 1 description: 'Position changed'.
	! !

!TreeZipperTest methodsFor: 'testing - copying' stamp: 'fbs 2/25/2011 08:06'!
testCopyReturnsExactCopy
	| copy zipper |
	zipper := (self makeZipperOn: self triangleTree) down.
	copy := zipper copy.
	
	self deny: copy == zipper.
	self assert: copy focus = zipper focus.
	self assert: copy up focus = zipper up focus.! !

!TreeZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 1/1/2011 15:55'!
testDeleteAtTopFails
	self should: [self emptyTree zipper delete] raise: ZipperNavigation.! !

!TreeZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 1/1/2011 16:06'!
testDeleteLeafNode
	| tree zipper |
	tree := ZTree value: 1 children: {ZTree value: 2. ZTree value: 3}.
	zipper := tree zipper down delete.
	
	self assert: zipper root = (ZTree value: 1 children: {ZTree value: 3}).! !

!TreeZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 1/1/2011 16:07'!
testDeleteLeafNodeDeletesNode
	| tree zipper |
	tree := ZTree value: 1 children: {ZTree value: 2. ZTree value: 3}.
	zipper := tree zipper down delete.
	
	self assert: zipper root = (ZTree value: 1 children: {ZTree value: 3}).! !

!TreeZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 1/1/2011 16:15'!
testDeleteLeafNodeFocusesOnParent
	| tree zipper |
	tree := ZTree value: 1 children: {ZTree value: 2}.
	zipper := tree zipper down.
	zipper := zipper delete.
	
	self assert: zipper root = (ZTree value: 1).
	self assert: zipper focus value = 1.! !

!TreeZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 1/1/2011 16:06'!
testDeleteNodeFocusesOnYoungerSibling
	| tree zipper |
	tree := ZTree value: 1 children: {ZTree value: 2. ZTree value: 3}.
	zipper := tree zipper down delete.
	
	self assert: zipper root = (ZTree value: 1 children: {ZTree value: 3}).! !

!TreeZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 1/1/2011 17:15'!
testDeleteOldestSiblingFocusesOnYoungerSibling
	| tree zipper |
	tree := ZTree value: 1 children: {ZTree value: 2. ZTree value: 3. ZTree value: 4}.
	zipper := tree zipper down.
	self assert: zipper focus value = 2.
	zipper := zipper delete.
	
	self assert: zipper focus value = 3 description: 'Zipper focused on wrong sibling'.
	self assert: zipper root = (ZTree value: 1 children: {ZTree value: 3. ZTree value: 4}) description: 'Wrong thing deleted'.! !

!TreeZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 1/1/2011 17:15'!
testDeleteYoungerSiblingFocusesOnOlderSibling
	| tree zipper |
	tree := ZTree value: 1 children: {ZTree value: 2. ZTree value: 3. ZTree value: 4}.
	zipper := tree zipper down right.
	self assert: zipper focus value = 3.
	zipper := zipper delete.
	
	self assert: zipper focus value = 2 description: 'Zipper focused on wrong sibling'.
	self assert: zipper root = (ZTree value: 1 children: {ZTree value: 2. ZTree value: 4}) description: 'Wrong thing deleted'.! !

!TreeZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 1/1/2011 17:16'!
testDeleteYoungestSiblingFocusesOnOlderSibling
	| tree zipper |
	tree := ZTree value: 1 children: {ZTree value: 2. ZTree value: 3. ZTree value: 4}.
	zipper := tree zipper down right right.
	self assert: zipper focus value = 4.
	zipper := zipper delete.
	
	self assert: zipper focus value = 3 description: 'Zipper focused on wrong sibling'.
	self assert: zipper root = (ZTree value: 1 children: {ZTree value: 2. ZTree value: 3}) description: 'Wrong thing deleted'.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 12/30/2010 22:18'!
testDownOnEmptyTreeSignalsException
	| zipper |
	zipper := TreeZipper on: self emptyTree.
	
	self
		should: [zipper down]
		raise: ZipperNavigation.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 12/30/2010 22:59'!
testDownOnLeafNodeSignalsException
	| zipper |
	zipper := TreeZipper on: self triangleTree.
	zipper := zipper down.

	self
		should: [zipper down]
		raise: ZipperNavigation.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 12/30/2010 23:44'!
testDownOnNonEmptyTreeFocusesOnFirstSibling
	| tree zipper |
	tree := self triangleTree.
	zipper := TreeZipper on: tree.
	zipper := zipper down.
	
	self assert: zipper focus = tree children first.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/16/2011 17:42'!
testDownOrElseCanUseFocus
	| node tree zipper |
	tree := self emptyTree.
	zipper := tree zipper.
	zipper downOrElse: [:focus | node := focus ].
	
	self assert: node = zipper focus.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/16/2011 07:50'!
testDownOrElseOnEmptyTreeRunsBlock
	| ran zipper |
	zipper := self emptyTree zipper.
	ran := false.
	zipper := zipper downOrElse: [ran := true].
	self assert: ran.
	self assert: zipper == ran description: 'Unexpected result'.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/13/2011 19:18'!
testDownOrElseOnLeafNodeRunsBlock
	| ran zipper |
	zipper := TreeZipper on: self triangleTree.
	zipper := zipper down.

	ran := false.
	zipper := zipper downOrElse: [ran := true].
	self assert: ran.
	self assert: zipper == ran description: 'Unexpected result'.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/13/2011 18:29'!
testDownOrElseOnNonEmptyTreeFocusesOnFirstSibling
	| ran tree zipper |
	tree := self triangleTree.
	zipper := TreeZipper on: tree.
	ran := false.
	zipper := zipper downOrElse: [ran := true].
	
	self assert: zipper focus = tree children first.
	self deny: ran.! !

!TreeZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 12/31/2010 23:46'!
testInsertDownInsertsChild
	| tree zipper |
	tree := ZTree value: 0.
	zipper := tree zipper.
	
	zipper := zipper insertDown: 1.
	"self should: [zipper up] raise: ZipperNavigation."
	self assert: zipper focus value = 1 description: 'Zipper not focused on new node'.
	self shouldnt: [zipper up] raise: ZipperNavigation.
	
	self assert: zipper root = (ZTree value: 0 children: {ZTree value: 1}).! !

!TreeZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 2/24/2011 17:29'!
testInsertDownOnEmptyTreeIsATreeWithOneNode
	| tree zipper |
	tree := ZTree empty.
	zipper := tree zipper.
	
	zipper := zipper insertDown: 0.
	self assert: zipper focus value = 0 description: 'Zipper not focused on new node'.
	self should: [zipper up] raise: ZipperNavigation.
	
	self assert: zipper root = (ZTree value: 0).! !

!TreeZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 1/1/2011 13:47'!
testInsertLeftInsertsImmediatelyOlderSibling
	| tree zipper |
	tree := ZTree value: 0 children: {ZTree value: 1. ZTree value: 2}.
	zipper := tree zipper.
	zipper := zipper down right insertLeft: 3.
	self assert: zipper root = (ZTree value: 0 children: {ZTree value: 1. ZTree value: 3. ZTree value: 2}).! !

!TreeZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 1/1/2011 13:20'!
testInsertLeftInsertsOlderSibling
	| tree zipper |
	tree := ZTree value: 0 children: {ZTree value: 1}.
	zipper := tree zipper.
	zipper := zipper down insertLeft: 2.
	self assert: zipper root = (ZTree value: 0 children: {ZTree value: 2. ZTree value: 1}).! !

!TreeZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 12/31/2010 23:52'!
testInsertRightInsertsYoungerSibling
	| tree zipper |
	tree := ZTree value: 0 children: {ZTree value: 1}.
	zipper := tree zipper.
	zipper := zipper down insertRight: 2.
	self assert: zipper root = (ZTree value: 0 children: {ZTree value: 1. ZTree value: 2}).! !

!TreeZipperTest methodsFor: 'testing - testing' stamp: 'fbs 2/17/2011 07:38'!
testIsEmptyOnEmptyTreeReturnsTrue
	self assert: self emptyTree zipper isEmpty.! !

!TreeZipperTest methodsFor: 'testing - testing' stamp: 'fbs 2/17/2011 07:42'!
testIsEmptyOnNonEmptyTreeAlwaysReturnsFalse
	| zipper |
	zipper := self triangleTree zipper.
	self deny: zipper isEmpty.
	
	self deny: zipper down isEmpty.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 12/31/2010 18:48'!
testLeftAtLeftmostNodeSignalsException
	| zipper |
	zipper := (TreeZipper on: self triangleTree) down.
	
	self
		should: [zipper left]
		raise: ZipperNavigation.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 12/31/2010 17:29'!
testLeftAtRootNodeSignalsException
	| zipper |
	zipper := TreeZipper on: self triangleTree.
	
	self
		should: [zipper left]
		raise: ZipperNavigation.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 12/31/2010 18:22'!
testLeftMovesToLeftSibling
	| tree zipper |
	tree := self nestedTriangleTree.
	zipper := TreeZipper on: tree.
	zipper := zipper down right down up.
	
	self assert: zipper focus value = 3 description: 'We followed the wrong path'.

	zipper := zipper left.
	self assert: zipper focus value = 2 description: 'Left didn''t go left'.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/13/2011 19:17'!
testLeftOrElseAtLeftmostNodeRunsBlock
	| ran zipper |
	zipper := (TreeZipper on: self triangleTree) down.
	
	zipper := zipper leftOrElse: [ran := true].
	
	self assert: ran.
	self assert: zipper == ran description: 'Unexpected result'.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/13/2011 19:17'!
testLeftOrElseAtRootNodeRunsBlock
	| ran zipper |
	zipper := TreeZipper on: self triangleTree.
	
	zipper := zipper leftOrElse: [ran := true].
	
	self assert: ran.
	self assert: zipper == ran description: 'Unexpected result'.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/16/2011 17:41'!
testLeftOrElseCanUseFocus
	| node tree zipper |
	tree := self emptyTree.
	zipper := tree zipper.
	zipper leftOrElse: [:focus | node := focus ].
	
	self assert: node = zipper focus.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/13/2011 18:34'!
testLeftOrElseMovesToLeftSibling
	| ran tree zipper |
	tree := self nestedTriangleTree.
	zipper := TreeZipper on: tree.
	zipper := zipper down right down up.
	
	self assert: zipper focus value = 3 description: 'We followed the wrong path'.

	ran := false.
	zipper := zipper leftOrElse: [ran := true].
	self assert: zipper focus value = 2 description: 'Left didn''t go left'.
	self deny: ran.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 1/1/2011 15:34'!
testLeftRightIdempotent
	| tree zipper |
	tree := self flatTree.
	zipper := tree zipper down.
	4 timesRepeat: [zipper := zipper right].
	self assert: zipper focus value = 6.
	
	"Node 2 has 4 younger siblings, so 4 is as far as we can navigate with #right.."
	1 to: 4 do:
		[ :i |
			| original navigated |
			original := navigated :=zipper.
			i timesRepeat: [ navigated := navigated left ].
			i timesRepeat: [ navigated := navigated right ].
			self assert: original focus = navigated focus ]! !

!TreeZipperTest methodsFor: 'testing - instance creation' stamp: 'fbs 12/30/2010 22:19'!
testOn
	| tree zipper |
	tree := self emptyTree.
	zipper := TreeZipper on: tree.
	
	self assert: zipper focus = tree.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 12/31/2010 18:49'!
testRightAtRightmostNodeSignalsException
	| zipper |
	zipper := (TreeZipper on: self triangleTree) down right.
	
	self
		should: [zipper right]
		raise: ZipperNavigation.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/13/2011 18:42'!
testRightAtRootNodeSignalsException
	| zipper |
	zipper := TreeZipper on: self triangleTree.
	
	self
		should: [zipper right]
		raise: ZipperNavigation.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 1/1/2011 15:35'!
testRightLeftIdempotent
	| tree zipper |
	tree := self flatTree.
	zipper := tree zipper down.
	
	"Node 2 has 4 younger siblings, so 4 is as far as we can navigate with #right.."
	1 to: 4 do:
		[ :i |
			| original navigated |
			original := navigated :=zipper.
			i timesRepeat: [ navigated := navigated right ].
			i timesRepeat: [ navigated := navigated left ].
			self assert: original focus = navigated focus ]! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 1/1/2011 15:27'!
testRightMovesToRightSibling
	| tree zipper |
	tree := self triangleTree.
	zipper := tree zipper.
	zipper := zipper down right.
	
	self assert: zipper focus = (tree children at: 2).! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/13/2011 19:15'!
testRightOrElseAtRightmostNodeRunsBlock
	| ran zipper |
	zipper := (TreeZipper on: self triangleTree) down right.
	
	ran := false.
	zipper := zipper rightOrElse: [ran := true].
	
	self assert: ran.
	self assert: zipper == ran description: 'Unexpected result'.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/13/2011 19:15'!
testRightOrElseAtRootNodeRunsBlock
	| ran zipper |
	zipper := TreeZipper on: self triangleTree.
	
	zipper := zipper rightOrElse: [ran := true].
	
	self assert: ran.
	self assert: zipper == ran description: 'Unexpected result'.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/16/2011 17:41'!
testRightOrElseCanUseFocus
	| node tree zipper |
	tree := self emptyTree.
	zipper := tree zipper.
	zipper rightOrElse: [:focus | node := focus ].
	
	self assert: node = zipper focus.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/13/2011 18:41'!
testRightOrElseMovesToRightSibling
	| ran tree zipper |
	tree := self triangleTree.
	zipper := tree zipper.
	zipper := zipper down.
	
	ran := false.
	zipper := zipper rightOrElse: [ran := true].
	
	self assert: zipper focus = (tree children at: 2).
	self deny: ran.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 1/1/2011 15:48'!
testRightStoresValueCorrectly
	"#right didn't store the value correctly, and would cause the right-most node
	value to be written to all the subsequently-visited nodes."
	| tree zipper |
	tree := self nestedTriangleTree.
	zipper := tree zipper.
	"Down, any (valid) combination of right/left, then up: no-op."
	self assert: zipper focus value = zipper down right up focus value.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 12/31/2010 23:31'!
testRootAtChildNodeReturnsRoot
	| tree zipper |
	tree := self triangleTree.
	zipper := tree zipper down.
	
	self assert: zipper root = tree.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 12/31/2010 23:29'!
testRootAtRootReturnsRoot
	| tree zipper |
	tree := self emptyTree.
	zipper := tree zipper.
	
	self assert: zipper root == tree.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 1/1/2011 15:26'!
testUpAtRootNodeSignalsException
	| zipper |
	zipper := self triangleTree zipper.
	
	self
		should: [zipper up]
		raise: ZipperNavigation.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/13/2011 19:14'!
testUpOrElseAtRootNodeRunsBlock
	| ran zipper |
	zipper := self triangleTree zipper.
	
	ran := false.
	zipper := zipper upOrElse: [ran := true.].
	
	self assert: ran.
	self assert: zipper == ran description: 'Unexpected result'.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/16/2011 17:41'!
testUpOrElseCanUseFocus
	| node tree zipper |
	tree := self emptyTree.
	zipper := tree zipper.
	zipper rightOrElse: [:focus | node := focus ].
	
	self assert: node = zipper focus.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/13/2011 19:20'!
testUpOrElseReturnsToParentNode
	| ran tree zipper |
	tree := self triangleTree.
	zipper := tree zipper.
	ran := false.
	zipper := zipper down upOrElse: [ran := true].
	
	self assert: zipper focus = tree.
	self deny: ran.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/13/2011 19:21'!
testUpOrElseReturnsToParentNode2
	| ran tree zipper |
	tree := ZTree value: 0 children: {ZTree value: 1 children: {ZTree value: 2}}.
	zipper := tree zipper.
	ran := false.
	zipper := zipper down down up upOrElse: [ran := true].
	
	self assert: zipper focus = tree.
	self deny: ran.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 1/1/2011 15:26'!
testUpReturnsToParentNode
	| tree zipper |
	tree := self triangleTree.
	zipper := tree zipper.
	zipper := zipper down up.
	
	self assert: zipper focus = tree.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 1/1/2011 14:46'!
testUpReturnsToParentNode2
	| tree zipper |
	tree := ZTree value: 0 children: {ZTree value: 1 children: {ZTree value: 2}}.
	zipper := tree zipper.
	zipper := zipper down down up up.
	
	self assert: zipper focus = tree.! !

!TreeZipperTest methodsFor: 'fixtures' stamp: 'fbs 12/31/2010 16:53'!
triangleTree
	" Best viewed in a monospaced font:
	   1
	  / \
	 2   3 "
	| b c |
	b := ZTree value: 2.
	c := ZTree value: 3.
	^ ZTree value: 1 children: {b. c.}! !

TestCase subclass: #ZTreeTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers-Tests'!

!ZTreeTests methodsFor: 'testing' stamp: 'fbs 4/24/2011 21:47'!
expectedFailures
	^ #(testAsDotFileOfDoubleNodeTreeShowsGraph.
	testAsDotFileOfEmptyTreeShowsEmptyGraph.
	testAsDotFileOfGrandchildTreeShowsGraph.
	testAsDotFileOfSingleNodeTreeShowsEmptyGraph)! !

!ZTreeTests methodsFor: 'testing - printing' stamp: 'fbs 4/23/2011 22:41'!
testAsDotFileOfDoubleNodeTreeShowsGraph
	self assert: ((ZTree value: 0 children: {ZTree value: 1}) asDotFile) = ('digraph {', (Character cr), '0->1', (Character cr), '}').! !

!ZTreeTests methodsFor: 'testing - printing' stamp: 'fbs 4/23/2011 22:39'!
testAsDotFileOfEmptyTreeShowsEmptyGraph
	self assert: (ZTree empty asDotFile) = ('digraph {', Character cr, '}').! !

!ZTreeTests methodsFor: 'testing - printing' stamp: 'fbs 4/23/2011 22:55'!
testAsDotFileOfGrandchildTreeShowsGraph
	| lines |
	lines := OrderedCollection with: 'digraph {'.
	lines add: '2->3'.
	lines add: '0->1'.
	lines add: '0->2'.
	lines add: '}'.
	
	self assert: ((ZTree value: 0 children: {ZTree value: 1. ZTree value: 2 children: {ZTree value: 3}}) asDotFile) = (lines reduce: [:acc :each | acc , Character cr, each]).! !

!ZTreeTests methodsFor: 'testing - printing' stamp: 'fbs 4/23/2011 22:39'!
testAsDotFileOfSingleNodeTreeShowsEmptyGraph
	self assert: ((ZTree value: 0) asDotFile) = ('digraph {', Character cr, '}').! !

!ZTreeTests methodsFor: 'testing - converting' stamp: 'fbs 4/27/2011 07:45'!
testAsMutableTreeEmptyTree
	| t |
	t := ZTree empty.
	self assert: t asMutableTree isNil.! !

!ZTreeTests methodsFor: 'testing - converting' stamp: 'fbs 4/27/2011 07:55'!
testAsMutableTreeMultipleChildren
	| t |
	t := (ZTree value: 0 children: {ZTree value: 1. ZTree value: 2}) asMutableTree.
	self assert: t notNil.
	self assert: t value = 0.
	self assert: t left notNil.
	self assert: t left value = 1.
	self assert: t left left isNil.
	self assert: t left right isNil.
	self assert: t right notNil.
	self assert: t right value = 2.
	self assert: t right left isNil.
	self assert: t right right isNil.! !

!ZTreeTests methodsFor: 'testing - converting' stamp: 'fbs 4/27/2011 07:53'!
testAsMutableTreeSingleChildLeft
	| t |
	t := (ZTree value: 0 children: {ZTree value: 1}) asMutableTree.
	self assert: t notNil.
	self assert: t value = 0.
	self assert: t left notNil.
	self assert: t left value = 1.
	self assert: t left left isNil.
	self assert: t left right isNil.
	self assert: t right isNil.! !

!ZTreeTests methodsFor: 'testing - converting' stamp: 'fbs 4/27/2011 17:47'!
testAsMutableTreeSingleChildRight
	| t |
	t := (ZTree value: 0 children: {ZTree empty. ZTree value: 1}) asMutableTree.
	self assert: t notNil.
	self assert: t value = 0.
	self assert: t left isNil.
	self assert: t right notNil.
	self assert: t right value = 1.
	self assert: t right left isNil.
	self assert: t right right isNil.! !

!ZTreeTests methodsFor: 'testing - converting' stamp: 'fbs 4/27/2011 07:47'!
testAsMutableTreeSingleNode
	| t |
	t := (ZTree value: 0) asMutableTree.
	self assert: t notNil.
	self assert: t value = 0.
	self assert: t left isNil.
	self assert: t right isNil.! !

!ZTreeTests methodsFor: 'testing - comparing' stamp: 'fbs 12/31/2010 23:35'!
testEqualsEqualSingleNodesReturnsTrue
	| a b |
	a := ZTree value: 1.
	b := ZTree value: 1.
	self assert: a = b.! !

!ZTreeTests methodsFor: 'testing - comparing' stamp: 'fbs 12/31/2010 23:36'!
testEqualsEqualTreesReturnsTrue
	| a b |
	a := ZTree value: 1 children: {ZTree value: 2}.
	b := ZTree value: 1 children: {ZTree value: 2}.
	self assert: a = b.! !

!ZTreeTests methodsFor: 'testing - comparing' stamp: 'fbs 12/31/2010 23:35'!
testEqualsUnequalSingleNodesReturnsFalse
	| a b |
	a := ZTree value: 1.
	b := ZTree value: 2.
	self deny: a = b.! !

!ZTreeTests methodsFor: 'testing - comparing' stamp: 'fbs 12/31/2010 23:37'!
testEqualsUnequalTreesInChildrenReturnsFalse
	| a b |
	a := ZTree value: 1 children: {ZTree value: 2}.
	b := ZTree value: 1 children: {ZTree value: 1}.
	self deny: a = b.! !

!ZTreeTests methodsFor: 'testing - comparing' stamp: 'fbs 12/31/2010 23:39'!
testEqualsUnequalTreesInValueReturnsFalse
	| a b |
	a := ZTree value: 1 children: {ZTree value: 2}.
	b := ZTree value: 2 children: {ZTree value: 2}.
	self deny: a = b.! !

!ZTreeTests methodsFor: 'testing - comparing' stamp: 'fbs 4/27/2011 17:45'!
testEqualsWithEmptyLeftSubtree
	| a b |
	a := ZTree value: 1 children: {ZTree empty. ZTree value: 2}.
	b := ZTree value: 1 children: {ZTree empty. ZTree value: 2}.
	self assert: a = b.! !

!ZTreeTests methodsFor: 'testing - comparing' stamp: 'fbs 4/27/2011 17:46'!
testEqualsWithEmptyRightSubtree
	| a b |
	a := ZTree value: 1 children: {ZTree value: 2. ZTree empty}.
	b := ZTree value: 1 children: {ZTree value: 2}.
	self assert: a = b.! !

!ZTreeTests methodsFor: 'testing - comparing' stamp: 'fbs 4/27/2011 17:44'!
testEqualsWithEmptySubtreeVersusNoChildren
	| a b |
	a := ZTree value: 1.
	b := ZTree value: 1 children: {ZTree empty}.
	self assert: a = b.! !

!ZTreeTests methodsFor: 'testing - printing' stamp: 'fbs 2/25/2011 21:33'!
testPrintEmptyTreeShowsSpecialSymbol
	self assert: '<>' = ZTree empty printString.! !

!ZTreeTests methodsFor: 'testing - printing' stamp: 'fbs 12/31/2010 23:32'!
testPrintSingleNodePrintsNoChildren
	self assert: (ZTree value: 1) printString = '(1)'.! !

!ZTreeTests methodsFor: 'testing - printing' stamp: 'fbs 12/31/2010 23:33'!
testPrintTreeNodePrintsChildren
	| t |
	t := ZTree value: 1 children: {ZTree value: 2. ZTree value: 3}.
	self assert: t printString = '(1 (2), (3))'.! !

!ZTreeTests methodsFor: 'testing - instance creation' stamp: 'fbs 2/25/2011 17:55'!
testValueWithEmptyTreeReturnsEmptyTree
	self assert: ZTree empty == (ZTree value: ZTree empty).! !

Object subclass: #BinarySearchTree
	instanceVariableNames: 'tree'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers-BST'!

!BinarySearchTree commentStamp: 'fbs 2/28/2011 22:27' prior: 0!
I implement a binary search tree in a strictly functional manner. I'm pretty naive, in that I do no balancing.!

!BinarySearchTree class methodsFor: 'instance creation' stamp: 'fbs 2/21/2011 22:12'!
tree: aTree
	^ self new tree: aTree.! !

!BinarySearchTree class methodsFor: 'instance creation' stamp: 'fbs 2/23/2011 21:04'!
with: anObject
	^ self new tree: (ZTree value: anObject).! !

!BinarySearchTree methodsFor: 'adding' stamp: 'fbs 2/21/2011 22:12'!
add: aComparableObject
	"Add some object that understands < to our tree. aComparableObject < node goes into the left subtree; otherwise, the object goes into the right subtree."
	^ BinarySearchTree tree: (self add: aComparableObject to: tree zipper)! !

!BinarySearchTree methodsFor: 'private' stamp: 'fbs 2/25/2011 22:56'!
add: anObject to: aZipper
	"If we're at a leaf node, and <, insertDown."
	"If we're not at a leaf node, move further into the tree."
	| currentValue leftChild move |
	aZipper focus isEmpty ifTrue: [ ^ (aZipper insertDown: anObject) root ].
	
	currentValue := aZipper focus value.
	
	"Move down to the left subnode."
	move := aZipper safeDown.
	move first = #success ifFalse:
		["Leaf node. If adding a right subtree, add the empty tree as a left subtree."
		^ (anObject < currentValue
			ifTrue: [ aZipper insertDown: anObject ]
			ifFalse: [ (aZipper insertDown: ZTree empty) insertRight: anObject ]) root ].

	leftChild := move second.
	
	"If less than, add to the left subtree."
	anObject < currentValue
		ifTrue: [ ^ self add: anObject to: leftChild ].
		
	"Otherwise, add to the right subtree."
	move := leftChild safeRight.
	^ self add: anObject to: move second.! !

!BinarySearchTree methodsFor: 'printing' stamp: 'fbs 4/24/2011 09:30'!
asDotFile
	^ tree asDotFile! !

!BinarySearchTree methodsFor: 'converting' stamp: 'fbs 2/28/2011 17:32'!
asOrderedCollection
	^ (self collect: [:each | each ]) asOrderedCollection.! !

!BinarySearchTree methodsFor: 'enumerating' stamp: 'fbs 2/28/2011 17:43'!
collect: aBlock
	^ InOrderTraversal new traverse: tree collecting: aBlock.! !

!BinarySearchTree methodsFor: 'initialize-release' stamp: 'fbs 2/24/2011 17:33'!
initialize
	tree := ZTree empty.! !

!BinarySearchTree methodsFor: 'printing' stamp: 'fbs 4/22/2011 22:13'!
printString
	^ tree printString.! !

!BinarySearchTree methodsFor: 'converting' stamp: 'fbs 4/27/2011 21:03'!
rebalanced
	^ self class tree: (tree asMutableTree balance asImmutableTree)! !

!BinarySearchTree methodsFor: 'private' stamp: 'fbs 2/21/2011 22:12'!
tree: aTree
	tree := aTree.! !

!BinarySearchTree methodsFor: 'accessing' stamp: 'fbs 2/28/2011 22:26'!
zipper
	"This lets me masquerade as a ZTree: PreOrderTraversal new traverse: aBST collecting: [:each | each * 2 ]."
	^ tree zipper.! !

Object subclass: #DepthTraversal
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers-Support'!

!DepthTraversal commentStamp: 'fbs 2/26/2011 23:06' prior: 0!
My subclasses define various ways of traversing trees and graphs.!

!DepthTraversal methodsFor: 'enumerating' stamp: 'fbs 4/24/2011 12:38'!
collect: aBlock using: aZipper
	^ self collectNodes: [:each | aBlock value: each value ] using: aZipper! !

!DepthTraversal methodsFor: 'private' stamp: 'fbs 2/18/2011 07:28'!
depthFirstTraversal: aZipper doing: aBlock
	self subclassResponsibility.! !

!DepthTraversal methodsFor: 'enumerating' stamp: 'fbs 4/24/2011 12:40'!
traverse: aTree collecting: aBlock
	"Traverse the VALUES in aTree."
	^ self traverseNodes: aTree collecting: [:each | aBlock value: each value].! !

!DepthTraversal methodsFor: 'enumerating' stamp: 'fbs 2/21/2011 07:24'!
traverse: aTree doing: aBlock
	^ self depthFirstTraversal: aTree zipper doing: aBlock.! !

!DepthTraversal methodsFor: 'enumerating' stamp: 'fbs 4/24/2011 12:37'!
traverseNodes: aTree collecting: aBlock
	"Traverse the NODES in aTree, letting you inspect the relationship between any node and its children."
	^ self collectNodes: aBlock using: aTree zipper.! !

DepthTraversal subclass: #InOrderTraversal
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers-Support'!

!InOrderTraversal commentStamp: 'fbs 2/26/2011 23:07' prior: 0!
In-order traversal only really makes sense for binary trees: with n-ary trees, "in order" means "process the first child, then the current node, then the remaining child nodes."!

!InOrderTraversal methodsFor: 'enumerating' stamp: 'fbs 4/24/2011 22:04'!
collectNodes: aBlock using: aZipper
	"A naive depth-first traversal. It fails in the presence of cycles."
	| collected left move right |
	aZipper focus isEmpty
		ifTrue: [ collected := #() asOrderedCollection ]
		ifFalse: [ collected := OrderedCollection with: (aBlock value: aZipper focus) ].
		
	move := aZipper safeDown.
	
	left := (move first = #success)
		ifTrue: [ self collectNodes: aBlock using: move second ]
		ifFalse: [ #() asOrderedCollection ].
	
	right := ((move first = #success) and: [(move := move second safeRight) first = #success])
		ifTrue: [self collectNodes: aBlock using: move second]
		ifFalse: [ #() asOrderedCollection ].

	^ left copy
		addAll: collected;
		addAll: right;
		yourself.! !

!InOrderTraversal methodsFor: 'private' stamp: 'fbs 2/25/2011 10:18'!
depthFirstTraversal: aZipper doing: aBlock
	^ (self collect: [:each | each] using: aZipper) do: aBlock.! !

DepthTraversal subclass: #PostOrderTraversal
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers-Support'!

!PostOrderTraversal commentStamp: 'fbs 2/26/2011 23:11' prior: 0!
I provide a post-order traversal of an n-tree: I traverse all child nodes before processing the current node.!

!PostOrderTraversal methodsFor: 'enumerating' stamp: 'fbs 4/24/2011 21:55'!
collectNodes: aBlock using: aZipper
	"A naive depth-first traversal. It fails in the presence of cycles."
	| collected move value |
	aZipper focus isEmpty
		ifTrue: [ collected := OrderedCollection new ]
		ifFalse:
			[value := aBlock value: aZipper focus.
			move := aZipper safeDown.
	
			collected := (move first = #success)
				ifTrue: [ (self collectNodes: aBlock using: move second) copy add: value; yourself ]
				ifFalse: [ { value } asOrderedCollection ]].
	
	move := aZipper safeRight.
	move first = #success ifTrue: [collected addAll: (self collectNodes: aBlock using: move second)].

	^ collected.! !

!PostOrderTraversal methodsFor: 'private' stamp: 'fbs 2/21/2011 18:01'!
depthFirstTraversal: aZipper doing: aBlock
	^ (self collect: [:each | each] using: aZipper) do: aBlock.
! !

DepthTraversal subclass: #PreOrderTraversal
	instanceVariableNames: 'root'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers-Support'!

!PreOrderTraversal commentStamp: 'fbs 2/13/2011 17:04' prior: 0!
I provide a pre-order traversal of an n-tree: I visit the value of a node and then its subchildren.!

!PreOrderTraversal methodsFor: 'enumerating' stamp: 'fbs 4/24/2011 21:56'!
collectNodes: aBlock using: aZipper
	"A naive depth-first traversal. It fails in the presence of cycles."
	| collected move value |
	aZipper focus isEmpty
		ifTrue: [ collected := OrderedCollection new ]
		ifFalse:
			[value := aBlock value: aZipper focus.
			move := aZipper safeDown.
	
			collected := (move first = #success)
				ifTrue: [ (self collectNodes: aBlock using: move second) copy addFirst: value; yourself ]
				ifFalse: [ { value } asOrderedCollection ]].
	
	move := aZipper safeRight.
	move first = #success ifTrue: [collected addAll: (self collectNodes: aBlock using: move second)].

	^ collected.! !

!PreOrderTraversal methodsFor: 'private' stamp: 'fbs 2/25/2011 17:37'!
depthFirstTraversal: aZipper doing: aBlock
	"A naive depth-first traversal. It fails in the presence of cycles."
	^ self collect: aBlock using: aZipper.
	
	"We're backtracking."
	"
	aZipper ifNil: [^ nil ].
	
	aBlock value: aZipper focus value.
	
	aZipper isEmpty
		ifTrue: [ ^ nil ].

	^ self depthFirstTraversal:
			(aZipper downOrElse:
				[self depthFirstTraversal:
					(aZipper rightOrElse: [
						self depthFirstTraversal:
							(aZipper up rightOrElse: [ nil ])
							doing: aBlock ])
					doing: aBlock ])
			doing: aBlock."! !

Object subclass: #ListZipper
	instanceVariableNames: 'focus trail'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers'!

!ListZipper commentStamp: 'fbs 12/28/2010 16:27' prior: 0!
A zipper is a structure that allows one, in a purely functional manner, to navigate a structure. It works by remembering the steps taken to focus on any particular part of a structure.

A ListZipper is a zipper for any sequenceable collection. It has two instance variables, namely

focus - the (sub)list on which we are currently focussed.
trail - the steps taken to focus on the focus instance variable.!

!ListZipper class methodsFor: 'instance creation' stamp: 'fbs 12/28/2010 16:48'!
on: aSequenceableCollection
	^ self new focus: aSequenceableCollection asOrderedCollection trail: OrderedCollection new.! !

!ListZipper methodsFor: 'mutating' stamp: 'fbs 12/30/2010 20:50'!
change: aBlock
	"Given an arity 1 block, replace the current value in the list with that of evaluating aBlock on that value."
	
	^ ListZipper new
		focus: (aBlock value: focus first) asOrderedCollection, focus allButFirst
		trail: trail.! !

!ListZipper methodsFor: 'accessing' stamp: 'fbs 12/28/2010 16:27'!
focus
	^ focus.! !

!ListZipper methodsFor: 'private' stamp: 'fbs 12/28/2010 16:36'!
focus: aSeqColl trail: anotherSeqColl
	focus := aSeqColl.
	trail := anotherSeqColl.! !

!ListZipper methodsFor: 'navigating' stamp: 'fbs 12/30/2010 23:22'!
left
	trail isEmpty ifTrue: [^ ZipperNavigation new signal: 'Left at the first element'].
	
	^ ListZipper new
		focus: (focus copy addFirst: trail first; yourself)
		trail: trail allButFirst.! !

!ListZipper methodsFor: 'navigating' stamp: 'fbs 12/30/2010 23:22'!
right
	focus isEmpty ifTrue: [^ ZipperNavigation new signal: 'Right at the last element'].
	
	^ ListZipper new
		focus: focus allButFirst
		trail: (trail copy addFirst: focus first; yourself)! !

Object subclass: #MutableZTree
	instanceVariableNames: 'children value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers-Support'!

!MutableZTree commentStamp: 'fbs 4/27/2011 22:21' prior: 0!
I implement a mutable version of ZTree. My sole reason for being is to provide a boundary within which mutable stuff can happen... like rebalancing a tree using the DSW algorithm.!

!MutableZTree methodsFor: 'converting' stamp: 'fbs 4/27/2011 17:30'!
asImmutableTree
	"Return an immutable version of self."
	self children isEmpty ifTrue: [^ ZTree value: self value].

	^ ZTree
		value: self value
		children: (self children
			collect: [:each | each
				ifNil: [ZTree empty]
				ifNotNil: [each asImmutableTree]]).! !

!MutableZTree methodsFor: 'private' stamp: 'fbs 4/27/2011 14:48'!
asTree: root ofSize: size
	"Convert my vine-self to a balanced tree."
	| sz leafCount internalNodeExp |
	internalNodeExp := ((size + 1) ln / 2 ln) floor.
	sz := size.
	
	leafCount := size + 1 - (2 raisedTo: internalNodeExp).
	self compress: root numbering: leafCount.
	sz := sz - leafCount.
	[sz > 1] whileTrue:
		[self compress: root numbering: sz / 2.
		sz := sz / 2].
	^ root.! !

!MutableZTree methodsFor: 'private' stamp: 'fbs 4/26/2011 22:22'!
asVine: root
	"The first half of the DSW algorithm. Turn self into a vine, and return the number of nodes in the vine."
	| vineTail remainder tempPtr size |
	vineTail := root.
	remainder := vineTail right.
	size := 0.
	[ remainder notNil ] whileTrue:
		[remainder left isNil
			ifTrue: "move vineTail down one"
				[vineTail := remainder.
				remainder := remainder right.
				size := size + 1.]
			ifFalse: "rotate"
				[tempPtr := remainder left.
				remainder left: tempPtr right.
				tempPtr right: remainder.
				remainder := tempPtr.
				vineTail right: tempPtr]].
	
	^ size.! !

!MutableZTree methodsFor: 'converting' stamp: 'fbs 4/27/2011 17:31'!
balance
	"Return myself, only balanced. Use the DSW algorithm."
	| pseudoroot size |
	pseudoroot := MutableZTree new
		right: self;
		yourself.
	size := self asVine: pseudoroot.
	pseudoroot := self asTree: pseudoroot ofSize: size.
	^ pseudoroot right.! !

!MutableZTree methodsFor: 'accessing' stamp: 'fbs 4/27/2011 07:51'!
children
	^ children.! !

!MutableZTree methodsFor: 'private' stamp: 'fbs 4/27/2011 15:18'!
compress: root numbering: count
	| scanner child |
	scanner := root.
	1 to: count do:
		[:i | child := scanner right.
		scanner right: child right.
		scanner := scanner right.
		child right: scanner left.
		scanner left: child].! !

!MutableZTree methodsFor: 'initialize-release' stamp: 'fbs 4/27/2011 12:11'!
initialize
	super initialize.
	"We have 'children' rather than 'left' and 'right' so that we have the same shape as ZTree."
	children := OrderedCollection new.! !

!MutableZTree methodsFor: 'accessing' stamp: 'fbs 4/27/2011 07:37'!
left
	^ children at: 1 ifAbsent: [nil].! !

!MutableZTree methodsFor: 'accessing' stamp: 'fbs 4/27/2011 12:14'!
left: aMutableBinaryNode
	children
		ifEmpty: [children addLast: aMutableBinaryNode]
		ifNotEmpty: [children at: 1 put: aMutableBinaryNode]! !

!MutableZTree methodsFor: 'printing' stamp: 'fbs 4/27/2011 07:41'!
printString
	^ '(', self value, ' ', (self left printString), ' ', (self right printString), ')'! !

!MutableZTree methodsFor: 'accessing' stamp: 'fbs 4/27/2011 07:37'!
right
	^ children at: 2 ifAbsent: [nil].! !

!MutableZTree methodsFor: 'accessing' stamp: 'fbs 4/27/2011 12:16'!
right: aMutableBinaryNode
	"If you set right before left, you need to make sure you set left to nil before continuing."
	[children size < 2] whileTrue: [children addLast: nil].
	children at: 2 put: aMutableBinaryNode.! !

!MutableZTree methodsFor: 'accessing' stamp: 'fbs 4/26/2011 21:43'!
value
	^ value.! !

!MutableZTree methodsFor: 'accessing' stamp: 'fbs 4/26/2011 21:43'!
value: anObject
	value := anObject.! !

Object subclass: #TreeZipper
	instanceVariableNames: 'focus trail'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers'!

!TreeZipper commentStamp: 'fbs 1/1/2011 18:01' prior: 0!
I am a zipper for navigating over a variable-arity tree, that is a tree that may have any number of children per node.

That is, I allow one to navigate arbitrarily over a tree structure (a ZTree), and change parts of it, where "change" means "return a structure that shares part of the original structure, but otherwise leaves the host structure unchanged".!

!TreeZipper class methodsFor: 'instance creation' stamp: 'fbs 12/31/2010 16:36'!
on: aTree
	^ self new focus: aTree trail: nil.! !

!TreeZipper methodsFor: 'mutating' stamp: 'fbs 2/9/2011 09:15'!
change: aBlock
	"aBlock is a unary block whose evaluated value we use to replace the current focus."
	^ self changeTo: (aBlock value: focus value).! !

!TreeZipper methodsFor: 'mutating' stamp: 'fbs 12/31/2010 19:32'!
changeTo: newValue
	^ TreeZipper new
		focus: (ZTree
			value: newValue
			children: focus children)
		trail: trail.! !

!TreeZipper methodsFor: 'copying' stamp: 'fbs 2/16/2011 07:45'!
copy
	^ TreeZipper new
		focus: self focus
		trail: self trail.! !

!TreeZipper methodsFor: 'mutating' stamp: 'fbs 1/1/2011 17:17'!
delete
	trail ifNil: [^ ZipperNavigation signal: 'Delete at top'].
	
	(trail leftNodes isEmpty and: [trail rightNodes isEmpty]) ifTrue:
		[^ TreeZipper new
			focus: (ZTree value: trail value) 
			trail: trail path].
			
	trail leftNodes isEmpty ifTrue:
		[^ TreeZipper new
			focus: trail rightNodes first
			trail: (TreeZipperNode
				path: trail path
				value: trail value
				leftNodes: trail leftNodes
				rightNodes: trail rightNodes allButFirst)].
			
	^ TreeZipper new
		focus: trail leftNodes first
		trail: (TreeZipperNode
			path: trail path
			value: trail value
			leftNodes: trail leftNodes allButFirst
			rightNodes: trail rightNodes).! !

!TreeZipper methodsFor: 'navigating' stamp: 'fbs 2/16/2011 17:33'!
down
	"Return a Zipper focusing on the first child element of this node."
	^ self move: #safeDown.! !

!TreeZipper methodsFor: 'navigating' stamp: 'fbs 2/16/2011 17:37'!
downOrElse: aBlock
	^ self move: #safeDown orElse: aBlock.! !

!TreeZipper methodsFor: 'accessing' stamp: 'fbs 12/30/2010 22:06'!
focus
	^ focus.! !

!TreeZipper methodsFor: 'private' stamp: 'fbs 12/31/2010 16:37'!
focus: aTree trail: aTreeZipperNode
	focus := aTree.
	trail := aTreeZipperNode.! !

!TreeZipper methodsFor: 'mutating' stamp: 'fbs 2/25/2011 17:52'!
insertDown: anObject
	"Insert anObject as a child node of the current focus. Sometimes you need a 'spacer' in your tre: a binary tree needs one so that the node can have just a right subtree. In that event, we insert a ZTree empty."

	^ focus isEmpty
		ifTrue: [self changeTo: anObject ]
		ifFalse: [ TreeZipper new
			focus: (anObject = ZTree empty
				ifTrue: [ ZTree empty ]
				ifFalse: [ ZTree value: anObject ])
			trail: (TreeZipperNode
				path: trail
				value: focus value
				leftNodes: OrderedCollection new
				rightNodes: focus children) ]! !

!TreeZipper methodsFor: 'mutating' stamp: 'fbs 1/1/2011 15:49'!
insertLeft: anObject
	trail ifNil: [^ ZipperNavigation signal: 'Insert at top'].
	^ TreeZipper new
		focus: focus
		trail: (TreeZipperNode
			path: trail path
			value: trail value
			leftNodes: (trail leftNodes copy addFirst: (ZTree value: anObject); yourself)
			rightNodes: trail rightNodes)! !

!TreeZipper methodsFor: 'mutating' stamp: 'fbs 1/1/2011 00:10'!
insertRight: anObject
	trail ifNil: [^ ZipperNavigation signal: 'Insert at top'].
	^ TreeZipper new
		focus: focus
		trail: (TreeZipperNode
			path: trail path
			value: trail value
			leftNodes: trail leftNodes
			rightNodes: (trail rightNodes copy addFirst: (ZTree value: anObject); yourself))! !

!TreeZipper methodsFor: 'testing' stamp: 'fbs 2/25/2011 17:35'!
isEmpty
	^ focus isEmpty.! !

!TreeZipper methodsFor: 'navigating' stamp: 'fbs 2/16/2011 17:34'!
left
	^ self move: #safeLeft.! !

!TreeZipper methodsFor: 'navigating' stamp: 'fbs 2/16/2011 17:36'!
leftOrElse: aBlock
	^ self move: #safeLeft orElse: aBlock.! !

!TreeZipper methodsFor: 'private' stamp: 'fbs 2/16/2011 17:43'!
move: directionSelector
	"Move in some direction. If we fall off the data structure, raise an exception."
	| move |
	move := self perform: directionSelector.
	move first = #success ifFalse: [ ^ ZipperNavigation signal: move first ].
	
	^ move second.! !

!TreeZipper methodsFor: 'private' stamp: 'fbs 2/16/2011 17:43'!
move: directionSelector orElse: aBlock
	"Move in some direction. If we can't, run aBlock. If aBlock takes parameters, pass in the current focus as an argument."
	| move |
	move := self perform: directionSelector.
	^ move first = #success
		ifTrue: [ move second ]
		ifFalse: [ aBlock cull: self focus ].! !

!TreeZipper methodsFor: 'navigating' stamp: 'fbs 2/16/2011 17:34'!
right
	^ self move: #safeRight.! !

!TreeZipper methodsFor: 'navigating' stamp: 'fbs 2/16/2011 17:36'!
rightOrElse: aBlock
	^ self move: #safeRight orElse: aBlock.! !

!TreeZipper methodsFor: 'navigating' stamp: 'fbs 12/31/2010 23:29'!
root
	"Return the possibly new/mutated tree under navigation"
	^ trail
		ifNil: [focus]
		ifNotNil: [self up root]! !

!TreeZipper methodsFor: 'navigating' stamp: 'fbs 2/28/2011 22:45'!
safeDown
	"Return a pair whose first element tells us whether we successfully went down (represented by the value #success), and a Zipper. If yes, a Zipper focusing on the first child element of this node. If not, a Zipper focused on where we are."
	| success zipper |
	success := self focus children isEmpty
		ifTrue: [ #downAtLeafNode ]
		ifFalse: [ #success ].
	
	zipper := (success = #success) ifTrue:
		[self class new
			focus: focus children first
			trail: (TreeZipperNode
					path: trail
					value: focus value
					leftNodes: OrderedCollection new
					rightNodes: focus children allButFirst)
		] ifFalse: [ self ].
	^ {success. zipper.}! !

!TreeZipper methodsFor: 'navigating' stamp: 'fbs 2/28/2011 22:46'!
safeLeft
	"Return a pair (successfully moved, zipper on new location)."
	| success zipper |
	trail
		ifNil: [ success := #leftAtRootNode ]
		ifNotNil: [trail leftNodes ifEmpty: [ success := #leftAtLeftmostSibling ]].
	success ifNil: [ success := #success ].
	
	zipper := (success = #success) ifTrue:
		[self class new
			focus: trail leftNodes first
			trail: (TreeZipperNode
				path: trail path
				value: trail value
				leftNodes: (trail leftNodes allButFirst)
				rightNodes: (trail rightNodes addFirst: focus; yourself))]
		ifFalse: [ self ].
	^ {success. zipper.}! !

!TreeZipper methodsFor: 'navigating' stamp: 'fbs 2/28/2011 22:46'!
safeRight
	"Return a pair (successfully moved, zipper on new location)."
	| success zipper |
	trail
		ifNil: [ success := #rightAtRootNode ]
		ifNotNil: [ trail rightNodes ifEmpty: [ success := #rightAtRightmostSibling ]].
	success ifNil: [ success := #success ].
	
	zipper := (success = #success)
		ifTrue: [ self class new
			focus: trail rightNodes first
			trail: (TreeZipperNode
				path: trail path
				value: trail value
				leftNodes: (trail leftNodes addFirst: focus; yourself)
				rightNodes: trail rightNodes allButFirst) ]
		ifFalse: [ self ].
		
	^ {success. zipper.}! !

!TreeZipper methodsFor: 'navigating' stamp: 'fbs 2/28/2011 22:46'!
safeUp
	"Return a pair (successfully moved, zipper on new location)."
	| success zipper |
	success := trail
		ifNil: [ #upAtRootNode ]
		ifNotNil: [ #success ].
	
	zipper := (success = #success)
		ifTrue: [ self class new
			focus: (ZTree
				value: trail value
				children: trail leftNodes reverse, focus asOrderedCollection, trail rightNodes)
			trail: trail path ]
		ifFalse: [ self ].
		
	^ {success. zipper.}! !

!TreeZipper methodsFor: 'accessing' stamp: 'fbs 2/16/2011 07:45'!
trail
	^ trail.! !

!TreeZipper methodsFor: 'navigating' stamp: 'fbs 2/16/2011 17:34'!
up
	^ self move: #safeUp.! !

!TreeZipper methodsFor: 'navigating' stamp: 'fbs 2/16/2011 17:35'!
upOrElse: aBlock
	^ self move: #safeUp orElse: aBlock.! !

Object subclass: #TreeZipperNode
	instanceVariableNames: 'leftNodes rightNodes path value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers'!

!TreeZipperNode commentStamp: 'fbs 1/1/2011 18:01' prior: 0!
I am a recursive structure that describes a path through a tree.

  self path

 refers to the previous position in the path, itself a TreeZipperNode. The first entry in a path has

  self path == nil

Being part of a zipper, I naturally represent a context-with-a-hole - that part of a tree minus the part currently under focus.!

!TreeZipperNode class methodsFor: 'instance creation' stamp: 'fbs 12/31/2010 17:07'!
path: aTreeZipperTrail value: anObject leftNodes: leftCollection rightNodes: rightCollection
	^ self new
		path: aTreeZipperTrail
		value: anObject
		leftNodes: leftCollection
		rightNodes: rightCollection.! !

!TreeZipperNode methodsFor: 'accessing' stamp: 'fbs 12/31/2010 00:19'!
leftNodes
	^ leftNodes! !

!TreeZipperNode methodsFor: 'accessing' stamp: 'fbs 12/31/2010 16:22'!
path
	^ path.! !

!TreeZipperNode methodsFor: 'private' stamp: 'fbs 12/31/2010 17:07'!
path: aTreeZipperTrail value: anObject leftNodes: leftCollection rightNodes: rightCollection
	path := aTreeZipperTrail.
	value := anObject.
	leftNodes := leftCollection.
	rightNodes := rightCollection.! !

!TreeZipperNode methodsFor: 'accessing' stamp: 'fbs 12/31/2010 00:19'!
rightNodes
	^ rightNodes.! !

!TreeZipperNode methodsFor: 'accessing' stamp: 'fbs 12/31/2010 17:06'!
value
	^ value.! !

Object subclass: #ZTree
	instanceVariableNames: 'children value'
	classVariableNames: 'Empty'
	poolDictionaries: ''
	category: 'Zippers-Support'!

!ZTree commentStamp: 'fbs 12/30/2010 23:32' prior: 0!
I'm a basic implementation of a tree, only present so that my package doesn't need to rely on a separate tree package.!

ZTree subclass: #ZEmptyTree
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers-Support'!

!ZEmptyTree commentStamp: 'fbs 2/22/2011 21:41' prior: 0!
I represent an empty tree.!

!ZEmptyTree methodsFor: 'accessing' stamp: 'fbs 2/22/2011 21:40'!
children
	^ #()! !

!ZEmptyTree methodsFor: 'testing' stamp: 'fbs 2/24/2011 17:26'!
isEmpty
	^ true.! !

!ZEmptyTree methodsFor: 'printing' stamp: 'fbs 2/25/2011 17:58'!
printString
	^ '<>'.! !

!ZEmptyTree methodsFor: 'accessing' stamp: 'fbs 2/22/2011 21:40'!
value
	^ nil.! !

!ZTree class methodsFor: 'instance creation' stamp: 'fbs 2/22/2011 21:41'!
empty
	^ Empty ifNil: [Empty := ZEmptyTree new].! !

!ZTree class methodsFor: 'instance creation' stamp: 'fbs 2/25/2011 17:53'!
value: anObject
	^ anObject = ZTree empty
		ifTrue: [ ZTree empty ]
		ifFalse: [ self new
			value: anObject
			children: OrderedCollection new ]! !

!ZTree class methodsFor: 'instance creation' stamp: 'fbs 12/30/2010 23:41'!
value: anObject children: aCollection
	^ self new value: anObject children: aCollection.! !

!ZTree methodsFor: 'comparing' stamp: 'fbs 4/27/2011 17:49'!
= anObject
	^ anObject class == self class and:
		[self value = anObject value and:
			[self childrenEqual: anObject children]].! !

!ZTree methodsFor: 'printing' stamp: 'fbs 4/24/2011 12:33'!
asDotFile
	| join |
	join := [:c | c
			ifEmpty: ['']
			ifNotEmpty: [ c reduce: [:s :e | s, ';', Character cr, e ]]].

	^ 'digraph {', Character cr,
		self printChildRecords, Character cr,
		(join value: (self associations collect: #printString)), Character cr,
		'}'! !

!ZTree methodsFor: 'printing' stamp: 'fbs 4/24/2011 20:56'!
asDotNode
	| labels labelString |
	labels := OrderedCollection new.
	(1 to: self children size do: [:i | labels addLast: '<p', i printString, '>']).
	labelString := labels ifEmpty: ['']
		ifNotEmpty: ['[label = "', (labels reduce: [:acc :each | acc , ' | ']), '"]'].
	^ '"node', self value printString, labelString, ';'! !

!ZTree methodsFor: 'converting' stamp: 'fbs 4/27/2011 17:31'!
asMutableTree
	"Return a mutable copy of this tree."
	| mutableCopy |
	self isEmpty ifTrue: [^nil].

	mutableCopy := MutableZTree new.
	mutableCopy value: self value.
	mutableCopy children addAll: (self children collect: #asMutableTree).
	^ mutableCopy.! !

!ZTree methodsFor: 'private' stamp: 'fbs 4/24/2011 10:35'!
associations
	^ self associationsWithLabels: [:node | node value].! !

!ZTree methodsFor: 'private' stamp: 'fbs 4/23/2011 22:00'!
associationsWithLabels: aBlock
	^ (self children
		inject: OrderedCollection new
		into: [:acc :each | acc addAll: each associations; yourself]) ,
	(self children collect: [:child | (aBlock value: self) -> (aBlock value: child)]).! !

!ZTree methodsFor: 'converting' stamp: 'fbs 4/26/2011 22:15'!
balance
	"Return a balanced version of self."
	^ self asMutableTree balance asImmutableTree.! !

!ZTree methodsFor: 'accessing' stamp: 'fbs 12/30/2010 23:33'!
children
	^ children! !

!ZTree methodsFor: 'private' stamp: 'fbs 4/27/2011 17:54'!
childrenEqual: aCollection
	"Compare this node's children against another collection of children. Note that a missing child is semantically identical to that child being the empty (sub)tree."
	1 to: (children size max: aCollection size) do:
		[:i | | left right |
		left := children at: i ifAbsent: [ZTree empty].
		right := aCollection at: i ifAbsent: [ZTree empty].
		(left = right) ifFalse: [^false]].
	^ true.! !

!ZTree methodsFor: 'initialize-release' stamp: 'fbs 12/30/2010 23:34'!
initialize
	super initialize.
	children := OrderedCollection new.! !

!ZTree methodsFor: 'testing' stamp: 'fbs 2/24/2011 17:26'!
isEmpty
	^ false.! !

!ZTree methodsFor: 'private' stamp: 'fbs 4/24/2011 12:48'!
printChildRecords
	^ (PreOrderTraversal new
		traverseNodes: self
		collecting: #asDotNode)
		reduce: [:acc :e | acc , Character cr, e].! !

!ZTree methodsFor: 'printing' stamp: 'fbs 12/30/2010 23:54'!
printString
	^ '(', self value printString,
		(children
			ifEmpty: ['']
			ifNotEmpty: [' ', (self children reduce: [:a :b | a printString, ', ', b printString])]),
		')'.! !

!ZTree methodsFor: 'accessing' stamp: 'fbs 12/31/2010 16:54'!
value
	^ value.! !

!ZTree methodsFor: 'private' stamp: 'fbs 12/31/2010 16:54'!
value: anObject children: aCollection
	value := anObject.
	children := aCollection asOrderedCollection.! !

!ZTree methodsFor: 'enumerating' stamp: 'fbs 12/31/2010 19:21'!
zipper
	^ TreeZipper on: self.! !
